using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Net.Sockets;

namespace ShoreSide.Core
{
    class Controller
    {
        private enum MessageTypes
        {
            DebugMessage = 251,
            CyclePacket = 250
        }

        public class StatisticsClass
        {
            public int Packets { get { return packets; } }
            public int LastCycleTime { get { return lastCycleTime; } }
            public int LatePackets { get { return latePackets; } }
            public int packets = 0;
            public int lastCycleTime = 0;
            public int latePackets = 0;
        }

        //New Data Available delegate
        public delegate void CycleHandler();
        public static CycleHandler Cycle = null;

        public delegate void TraceHandler(string s);
        public static TraceHandler Trace = null;

        public delegate void StatusHandler(string s);
        public static StatusHandler StatusMessage = null;

        private static TcpClient tcp = null;
        private static NetworkStream stream = null;

        public static Decimal PowerSupplyVoltage = 0;

        private static bool ShuttingDown = false;
        public static bool PowerOffPending = false;
        public static bool ResetPending = false;
        public static bool BootLoadPending = false;

        private static Thread UpdateThread = null;

        public static Properties.Settings Settings = new ShoreSide.Properties.Settings();

        public static StatisticsClass Statistics = new StatisticsClass();

        private const int LATE_PACKET = 28;


        public struct CyclePacket
        {
            public Int16[] Positions;
            public byte[] Current;
            public byte[] CurrentCycles;
            public byte Voltage;
            public byte[] Feet;
        }

        public struct UpdatePacket
        {
            public Int16[] Positions;
        }

        public static CyclePacket LastCyclePacket;
        public static UpdatePacket OutgoingPacket;

        static Controller()
        {
            UpdateThread = new Thread(new ThreadStart(UpdateCycleThread));
            UpdateThread.Start();
        }

        private static byte[] ReadNetwork(int bytes)
        {
            int bytesRead = 0;
            byte[] incoming = new byte[bytes];

            while (bytesRead < bytes)
                bytesRead += stream.Read(incoming, bytesRead, bytes - bytesRead);

            return incoming;
        }

        public static void UpdateCycleThread()
        {
            bool LastPacketGood = false;
            bool PacketLate = false;

            Timer CycleTimer = new Timer();
            List<byte> HeaderFlags = new List<byte>();
            tcp = new TcpClient();

            while (!ShuttingDown)
            {
                while (!tcp.Connected && !ShuttingDown)
                {
                    try
                    {
                        if (tcp != null)
                            tcp.Close();

                        tcp = new TcpClient();
                        tcp.NoDelay = true;
                        tcp.ReceiveTimeout = 5000;

                        StatusMessage("Connecting to " + Settings.IPAddress + "...");
                        tcp.Connect(Settings.IPAddress, Settings.TCPPort);
                        stream = tcp.GetStream();
                        StatusMessage("Connected!");
                    }
                    catch (Exception e)
                    {
                        StatusMessage("Connection Failed ... " + e.Message);
                        Thread.Sleep(500);
                    }
                }

                if (tcp.Connected)
                {
                    if (!BootLoader.ReadyToLoad)
                    {
                        try
                        {
                            while (HeaderFlags.Count < 4 && stream.DataAvailable)
                                HeaderFlags.Add(ReadNetwork(1)[0]);

                            if (TestHeader(HeaderFlags))
                            {
                                HeaderFlags.Clear();

                                switch ((MessageTypes)ReadNetwork(1)[0])
                                {
                                    case MessageTypes.CyclePacket:

                                        CycleTimer.Stop();
                                        Statistics.lastCycleTime = CycleTimer.Duration;
                                        if (Statistics.lastCycleTime > LATE_PACKET)
                                        {
                                            PacketLate = true;
                                            Statistics.latePackets++;
                                        }
                                        else
                                            PacketLate = false;

                                        Statistics.packets++;
                                        CycleTimer.Start();

                                        CyclePacket packet = ReadCyclePacket();
                                        ReadFooter();
                                        OutgoingPacket.Positions = new Int16[18];

                                        LastCyclePacket = packet;
                                        if (Cycle != null)
                                            Cycle();

                                        if (!PacketLate)
                                        {
                                            if (ResetPending || BootLoadPending)
                                            {
                                                stream.Write(OneBytePacket((byte)'r'), 0, 5);
                                                stream.Flush();
                                                ResetPending = false;
                                                if (BootLoadPending)
                                                {
                                                    BootLoadPending = false;
                                                    BootLoader.ReadyToLoad = true;
                                                }
                                            }
                                            else if (PowerOffPending)
                                            {
                                                stream.Write(OneBytePacket((byte)'o'), 0, 5);
                                                PowerOffPending = false;
                                            }
                                            else
                                                WriteOutgoingPacket();
                                        }
                                        PowerSupplyVoltage = (Decimal)((int)(packet.Voltage / 9.15 * 100) / (float)100);

                                        break;

                                    case MessageTypes.DebugMessage:
                                        //Length prefixed string
                                        byte[] message = ReadNetwork(ReadNetwork(1)[0]);
                                        Trace("DBG: " + System.Text.Encoding.ASCII.GetString(message));
                                        ReadFooter();
                                        break;
                                }
                                if (!LastPacketGood)
                                    Trace("Stream Synchronized");

                                LastPacketGood = true;
                            }
                            else
                            {
                                if (HeaderFlags.Count == 4)
                                {
                                    if (LastPacketGood)
                                        Trace("Stream Synchronization Error");

                                    LastPacketGood = false;
                                    HeaderFlags.RemoveAt(0);
                                }
                                else
                                    Thread.Sleep(5);
                            }
                        }
                        catch (Exception e)
                        {
                            Trace("ERROR: Exception in CycleThread: " + e.Message);
                        }
                    }
                    else
                    {
                        Thread.Sleep(500);
                        Trace("BOOT: Connecting");

                        if (BootLoader.Connect(stream))
                        {
                            Trace("BOOT: Writing Hex");
                            BootLoader.WriteHex(stream);
                            Trace("BOOT: Firmware Update Success");
                        }
                        else
                            Trace("BOOT: Failed to Connect");

                        BootLoader.ReadyToLoad = false;
                    }
                }
            }
        }

        private static bool TestHeader(List<byte> HeaderFlags)
        {
            if (HeaderFlags.Count < 4)
                return false;
            else
                return HeaderFlags[0] == 255 &&
                    HeaderFlags[1] == 254 &&
                    HeaderFlags[2] == 253 &&
                    HeaderFlags[3] == 252;
        }

        private static void ReadFooter()
        {
            ReadNetwork(2);
        }

        public static void ShutDown()
        {
            ShuttingDown = true;
            UpdateThread.Abort();
        }

        private static CyclePacket ReadCyclePacket()
        {
            const int BYTE_COUNT = 64;

            CyclePacket packet = new CyclePacket();
            packet.Positions = new Int16[18];
            packet.Current = new byte[18];
            packet.CurrentCycles = new byte[3];
            packet.Feet = new byte[6];

            byte[] b = ReadNetwork(BYTE_COUNT);
            int BPointer = 0;

            for (int i = 0; i < 18; i++)
            {
                packet.Positions[i] = BitConverter.ToInt16(b, i * 2);
            }
            BPointer += 18 * 2;

            for (int i = 0; i < 18; i++)
                packet.Current[i] = b[BPointer + i];
            BPointer += 18;

            packet.CurrentCycles[0] = b[BPointer + 0];
            packet.CurrentCycles[1] = b[BPointer + 1];
            packet.CurrentCycles[2] = b[BPointer + 2];
            BPointer += 3;

            packet.Voltage = b[BPointer + 0];
            BPointer++;

            for (int i = 0; i < 6; i++)
                packet.Feet[i] = b[BPointer + i];
            BPointer += 6;

            return packet;
        }

        private static void WriteOutgoingPacket()
        {
            byte[] b = new byte[41];
            b[0] = 255;
            b[1] = 254;
            b[2] = 253;
            b[3] = 252;
            b[4] = (byte)'t';

            int i = 5;

            foreach (Int16 pos in OutgoingPacket.Positions)
            {
                b[i++] = (byte)(pos % 256);
                b[i++] = (byte)(pos / 256);
            }

            stream.Write(b, 0, b.Length);
            stream.Flush();
        }

        private static byte[] OneBytePacket(byte b)
        {
            return new byte[] { 255, 254, 253, 252, b };
        }
    }
}
