using System;
using System.Collections.Generic;
using System.Text;

namespace ShoreSide.Core
{
    public class Servo : IDisposable
    {
        public static List<Servo> AllServos = new List<Servo>();
        public static Servo[] ServosByIndex = new Servo[18];

        protected int servoIndex = 0;
        protected int currentReading = 0;
        protected int positionReading = 0;

        protected int RawPosition = 0;

        public bool DisableOffset = false;
        
        public int CalibrationOffset = 0;
        public double CalibrationReadingOffset = 0;
        public double CalibrationReadingRatio = 0;

        public bool Idle = true;

        private Controller.CycleHandler cycleDelegate = null;

        public static void Initialize()
        {
            for (int i = 0; i < 18; i++)
            {
                Servo s = new Servo(i);
            }

            ServosByIndex[0].CalibrationReadingOffset = 52;
            ServosByIndex[0].CalibrationReadingRatio = 4.138;
            ServosByIndex[1].CalibrationReadingOffset = 17;
            ServosByIndex[1].CalibrationReadingRatio = 4.334;
            ServosByIndex[2].CalibrationReadingOffset = 97;
            ServosByIndex[2].CalibrationReadingRatio = 4.301;
            ServosByIndex[3].CalibrationReadingOffset = 55;
            ServosByIndex[3].CalibrationReadingRatio = 4.223;
            ServosByIndex[4].CalibrationReadingOffset = 23;
            ServosByIndex[4].CalibrationReadingRatio = 4.473;
            ServosByIndex[5].CalibrationReadingOffset = 83;
            ServosByIndex[5].CalibrationReadingRatio = 4.400;

            ServosByIndex[6].CalibrationReadingOffset = 95;
            ServosByIndex[6].CalibrationReadingRatio = 4.337;
            ServosByIndex[7].CalibrationReadingOffset = 20;
            ServosByIndex[7].CalibrationReadingRatio = 4.241;
            ServosByIndex[8].CalibrationReadingOffset = 77;
            ServosByIndex[8].CalibrationReadingRatio = 4.096;
            ServosByIndex[9].CalibrationReadingOffset = 62;
            ServosByIndex[9].CalibrationReadingRatio = 4.224;
            ServosByIndex[10].CalibrationReadingOffset = 16;
            ServosByIndex[10].CalibrationReadingRatio = 4.292;
            ServosByIndex[11].CalibrationReadingOffset = 67;
            ServosByIndex[11].CalibrationReadingRatio = 4.155;

            ServosByIndex[12].CalibrationReadingOffset = 84;
            ServosByIndex[12].CalibrationReadingRatio = 4.255;
            ServosByIndex[13].CalibrationReadingOffset = 7;
            ServosByIndex[13].CalibrationReadingRatio = 4.334;
            ServosByIndex[14].CalibrationReadingOffset = 39;
            ServosByIndex[14].CalibrationReadingRatio = 3.972;
            ServosByIndex[15].CalibrationReadingOffset = 56;
            ServosByIndex[15].CalibrationReadingRatio = 4.289;
            ServosByIndex[16].CalibrationReadingOffset = 20;
            ServosByIndex[16].CalibrationReadingRatio = 4.430;
            ServosByIndex[17].CalibrationReadingOffset = 61;
            ServosByIndex[17].CalibrationReadingRatio = 4.210;

            ServosByIndex[Body.ServoMap(0, Body.Joints.ShoulderYaw)].CalibrationOffset = 150;
            ServosByIndex[Body.ServoMap(0, Body.Joints.ShoulderPitch)].CalibrationOffset = 264;
            ServosByIndex[Body.ServoMap(0, Body.Joints.Elbow)].CalibrationOffset = 140;
            ServosByIndex[Body.ServoMap(1, Body.Joints.ShoulderYaw)].CalibrationOffset = 160;
            ServosByIndex[Body.ServoMap(1, Body.Joints.ShoulderPitch)].CalibrationOffset = 124;
            ServosByIndex[Body.ServoMap(1, Body.Joints.Elbow)].CalibrationOffset = 4;
            ServosByIndex[Body.ServoMap(2, Body.Joints.ShoulderYaw)].CalibrationOffset = 100;
            ServosByIndex[Body.ServoMap(2, Body.Joints.ShoulderPitch)].CalibrationOffset = 224;
            ServosByIndex[Body.ServoMap(2, Body.Joints.Elbow)].CalibrationOffset = 104;
            ServosByIndex[Body.ServoMap(3, Body.Joints.ShoulderYaw)].CalibrationOffset = 210;
            ServosByIndex[Body.ServoMap(3, Body.Joints.ShoulderPitch)].CalibrationOffset = 270;
            ServosByIndex[Body.ServoMap(3, Body.Joints.Elbow)].CalibrationOffset = -2;
            ServosByIndex[Body.ServoMap(4, Body.Joints.ShoulderYaw)].CalibrationOffset = 450;
            ServosByIndex[Body.ServoMap(4, Body.Joints.ShoulderPitch)].CalibrationOffset = 232;
            ServosByIndex[Body.ServoMap(4, Body.Joints.Elbow)].CalibrationOffset = 146;
            ServosByIndex[Body.ServoMap(5, Body.Joints.ShoulderYaw)].CalibrationOffset = 140;
            ServosByIndex[Body.ServoMap(5, Body.Joints.ShoulderPitch)].CalibrationOffset = 230;
            ServosByIndex[Body.ServoMap(5, Body.Joints.Elbow)].CalibrationOffset = 84;

            foreach (Servo s in AllServos)
                s.Angle = 0;
        }

        public Servo(int servoIndex)
        {
            this.servoIndex = servoIndex;
            
            cycleDelegate = new Controller.CycleHandler(Cycle);
            Controller.Cycle += cycleDelegate;
            lock (AllServos)
            {
                AllServos.Add(this);
                ServosByIndex[ServoIndex] = this;
            }
        }

        public Servo(int leg, Body.Joints Joint)
            : this(Body.ServoMap(leg, Joint))
        {
        }

        public void Cycle()
        {
            currentReading = Controller.LastCyclePacket.Current[servoIndex];
            positionReading = (int)((Controller.LastCyclePacket.Positions[ServoIndex] + -1.4*currentReading) * CalibrationReadingRatio - CalibrationReadingOffset);

            this.OnCycle();
        }

        protected virtual void OnCycle()
        {
            Controller.OutgoingPacket.Positions[this.servoIndex] = (Int16)(this.Idle ? 0 : this.RawPosition);
        }

        public double Angle
        {
            set
            {
                Position = (int)(2300 + value * 16.66);
            }
            get
            {
                return (Position - 2300) / 16.66;
            }
        }

        public double AngleReading
        {
            get
            {
                return (PositionReading - 2300) / 16.66;
            }
        }

        private int Position
        {
            get
            {
                if (DisableOffset)
                    return RawPosition;
                else
                    return (RawPosition - CalibrationOffset);
            }
            set
            {
                if (DisableOffset)
                    RawPosition = value;
                else
                    RawPosition = (value + CalibrationOffset);
            }
        }

        public int PositionReading
        {
            get
            {
                if (DisableOffset)
                    return (int)positionReading;
                else
                    return (int)(positionReading - CalibrationOffset);
            }
        }

        public int ServoIndex { get { return servoIndex; } }
        public int CurrentReading { get { return currentReading; } }

        void IDisposable.Dispose()
        {
            Controller.Cycle -= cycleDelegate;
            lock (AllServos)
            {
                AllServos.Remove(this);
                ServosByIndex[ServoIndex] = null;
            }

        }
    }
}