using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ShoreSide.Core;
using Sharp3D.Math.Core;

namespace ShoreSide.UI
{

    public partial class JointEnable : SSForm
    {
        public class JointModel : UserControl
        {
            private const int ButtonSize = 12;
            private Body Posture = Body.FromReadings();
            struct ServoEnable
            {
                public float X;
                public float Y;
                public Servo Servo;
                public bool HitTest(int x, int y, int Size)
                {
                    if (x >= X - Size / 2 && x <= X + Size / 2 && y >= Y - Size / 2 && y <= Y + Size / 2)
                        return true;
                    else
                        return false;
                }
            }

            private ServoEnable[] ServoEnables = new ServoEnable[18];

            public JointModel()
            {
                Recalc();
            }

            protected override void OnResize(EventArgs e)
            {
                Recalc();
                base.OnResize(e);
            }

            private void Recalc()
            {
                foreach (Leg l in Posture.Legs)
                    l.SetAngles(0, 0, 0);

                int i = 0;
                foreach (Leg l in Posture.Legs)
                {

                    Vector3F ShoulderYaw = l.LegLocalToWorld(l.ShoulderYawPosition);
                    Vector3F ShoulderPitch = l.LegLocalToWorld(l.ShoulderPitchPosition);
                    Vector3F Elbow = l.LegLocalToWorld(l.ElbowPosition);

                    ServoEnables[i].X = ShoulderYaw.X;
                    ServoEnables[i].Y = ShoulderYaw.Z;
                    ServoEnables[i].Servo = Servo.ServosByIndex[l.ShoulderYawIndex];
                    i++;

                    ServoEnables[i].X = ShoulderPitch.X;
                    ServoEnables[i].Y = ShoulderPitch.Z;
                    ServoEnables[i].Servo = Servo.ServosByIndex[l.ShoulderPitchIndex];
                    i++;

                    ServoEnables[i].X = Elbow.X;
                    ServoEnables[i].Y = Elbow.Z;
                    ServoEnables[i].Servo = Servo.ServosByIndex[l.ElbowIndex];
                    i++;
                }

                //Scale the coords to our picturebox
                for (i = 0; i < 18; i++)
                {
                    ServoEnables[i].X /= 2;
                    ServoEnables[i].Y /= 2;
                    ServoEnables[i].X += this.Width / 2;
                    ServoEnables[i].Y += this.Height / 2;
                }
            }

            protected override void OnPaintBackground(PaintEventArgs e)
            {
                //base.OnPaintBackground(e);
            }

            protected override void OnMouseClick(MouseEventArgs e)
            {
                foreach (ServoEnable se in ServoEnables)
                    if (se.HitTest(e.X, e.Y, ButtonSize))
                    {
                        se.Servo.Idle = !se.Servo.Idle;
                        break;
                    }
                
                base.OnMouseClick(e);
            }
            protected override void OnPaint(PaintEventArgs e)
            {
                Graphics g = Graphics.FromHwnd(this.Handle);

                using (g)
                {
                    g.FillRectangle(Brushes.White, this.ClientRectangle);
                    foreach (ServoEnable se in ServoEnables)
                    {
                        g.FillRectangle(se.Servo.Idle ? Brushes.Red : Brushes.Green, se.X - ButtonSize/2, se.Y - ButtonSize / 2, ButtonSize, ButtonSize);
                        g.DrawRectangle(Pens.Black, se.X - ButtonSize / 2, se.Y - ButtonSize / 2, ButtonSize, ButtonSize);
                    }
                }
            }
        }

        private JointModel model = new JointModel();

        public JointEnable()
        {
            InitializeComponent();

            model.Location = new Point(0, 0);
            model.Width = this.ClientRectangle.Width;
            model.Height = this.ClientRectangle.Height;

            this.Controls.Add(model);
            model.Visible = true;
        }

        private void tmr_Tick(object sender, EventArgs e)
        {
            model.Refresh();
        }
    }
}