using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ShoreSide.Core;
using System.Collections;

namespace ShoreSide.UI
{
    public partial class MDIMain : Form
    {
        private string StatusMessage = null;
        private Queue<String> TraceQueue = new Queue<string>();
        private int TraceCounter = 0;

        public MDIMain()
        {
            InitializeComponent();
            this.Show();
            this.Disposed += new EventHandler(MDIMain_Disposed);
            Controller.StatusMessage += new Controller.StatusHandler(UpdateStatus);
            Controller.Trace += new Controller.TraceHandler(Trace);

            Servo.Initialize();
                
        }

        void MDIMain_Disposed(object sender, EventArgs e)
        {
            Controller.ShutDown();
        }

        private void UpdateStatus(string Message)
        {
            StatusMessage = Message;
            Trace("STATUS: " + Message);
        }

        private void timerStatus_Tick(object sender, EventArgs e)
        {
            toolStripStatusLabel.Text = StatusMessage;
            toolStripStatusPower.Text = Controller.PowerSupplyVoltage.ToString("0.00V");

            bool scroll = listTrace.SelectedIndex == listTrace.Items.Count - 1;
            while (TraceQueue.Count > 0)
                listTrace.Items.Add(TraceQueue.Dequeue());

            if (scroll)
                listTrace.SelectedIndex = listTrace.Items.Count - 1;
            
        }

        private void traceWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            traceWindowToolStripMenuItem.Checked = !traceWindowToolStripMenuItem.Checked;
            panelTrace.Visible = traceWindowToolStripMenuItem.Checked;

        }

        private void Trace(string s)
        {
            TraceQueue.Enqueue((TraceCounter++).ToString("00000: ") + s);
        }

        private void modelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Model.ShowForm(this, typeof(Model));
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void statisticsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Stats.ShowForm(this, typeof(Stats));
        }

        private void toolPowerOff_Click(object sender, EventArgs e)
        {
            Controller.PowerOffPending = true;
        }

        private void toolReset_Click(object sender, EventArgs e)
        {
            Controller.ResetPending = true;
        }

        private void firmwareToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FirmwareUpdate.ShowForm(this, typeof(FirmwareUpdate));
        }

        private void configToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Config.ShowForm(this, typeof(Config));
        }

        private void wiPortConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://" + Controller.Settings.IPAddress);
        }

        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Test.ShowForm(this, typeof(Test));
        }

        private void PCTPerceptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PCTPerceptions.ShowForm(this, typeof(PCTPerceptions));
        }

        private void toolIdle_Click(object sender, EventArgs e)
        {
            foreach (Servo s in Servo.AllServos)
                s.Idle = true;
        }

        private void toolActive_Click(object sender, EventArgs e)
        {
            foreach (Servo s in Servo.AllServos)
                s.Idle = false;
        }

        private void toolCenter_Click(object sender, EventArgs e)
        {
            foreach (Servo s in Servo.AllServos)
                s.Angle = 0;
        }

        private void feetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Feet.ShowForm(this, typeof(Feet));
        }

        private void jointEnableDisableToolStripMenuItem_Click(object sender, EventArgs e)
        {
            JointEnable.ShowForm(this, typeof(JointEnable));
        }

        private void toolComply_Click(object sender, EventArgs e)
        {
            foreach (Servo s in Servo.ServosByIndex)
                s.Angle = s.AngleReading;
        }
    }

    public class SSForm : Form
    {
        protected static ArrayList Instances = new ArrayList();

        public static void ShowForm(Form mdiParent, Type t)
        {
            SSForm form = null;

            foreach (object o in Instances)
                if (o.GetType() == t)
                    form = (SSForm)o;
            if (form == null)
            {
                form = (SSForm)System.Activator.CreateInstance(t);
                Instances.Add(form);
            }

            form.MdiParent = mdiParent;
            form.Show();
            form.BringToFront();
        }

        protected override void OnClosed(EventArgs e)
        {
            Instances.Remove(this);
            base.OnClosed(e);
        }
    }
}
