using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace ShoreSide.UI
{
    public partial class Meter : UserControl
    {
        private double max = 100;
        private double min = 0;
        private double currentValue = 0;
        private double thresholdValue = 0;

        private Color meterColor = Color.Blue;
        private Color markerColor = Color.Black;
        private Color thresholdColor = Color.Red;

        public double Max
        {
            set { max = value; }
            get { return max; }
        }

        public double Min
        {
            set { min = value; }
            get { return min; }
        }

        public Color MeterColor
        {
            get
            {
                return meterColor;
            }
            set
            {
                meterColor = value;
            }
        }

        public Color MarkerColor
        {
            get
            {
                return markerColor;
            }
            set
            {
                markerColor = value;
            }
        }

        public Color ThresholdColor
        {
            get
            {
                return thresholdColor;
            }
            set
            {
                thresholdColor = value;
            }
        }

        public double Value
        {
            get
            {
                return currentValue;
            }
            set
            {
                currentValue = value;
                this.Invalidate();
            }
        }

        public double ThresholdValue
        {
            get
            {
                return thresholdValue;
            }
            set
            {
                thresholdValue = value;
                this.Invalidate();
            }
        }

        public Meter()
        {
            InitializeComponent();
        }

        private void Redraw()
        {
            SolidBrush backBrush = new SolidBrush(this.BackColor);
            SolidBrush meterBrush = new SolidBrush(this.MeterColor);
            Pen markerPen = new Pen(markerColor, 2);
            SolidBrush thresholdBrush = new SolidBrush(this.ThresholdColor);

            Graphics g = Graphics.FromHwnd(this.Handle);
            using (g)
            {
                int meterPixel = ValueToPixel(Value);
                int markerPixel = ValueToPixel(ThresholdValue);
                g.FillRectangle(backBrush, this.ClientRectangle.Left, this.ClientRectangle.Top, this.ClientRectangle.Width, meterPixel);
                g.FillRectangle(meterBrush, this.ClientRectangle.Left, this.ClientRectangle.Top + meterPixel, this.ClientRectangle.Width, this.ClientRectangle.Height - meterPixel);
                if (Value > ThresholdValue)
                    g.FillRectangle(thresholdBrush, this.ClientRectangle.Left, this.ClientRectangle.Top + meterPixel, this.ClientRectangle.Width, markerPixel - meterPixel);

                g.DrawLine(markerPen, this.ClientRectangle.Left, this.ClientRectangle.Top + markerPixel, this.ClientRectangle.Left + this.ClientRectangle.Width, this.ClientRectangle.Top + markerPixel);
            }

            backBrush.Dispose();
            meterBrush.Dispose();
            markerPen.Dispose();
            thresholdBrush.Dispose();
        }
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //base.OnPaintBackground(e);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Redraw();
        }

        private int ValueToPixel(double value)
        {
            double Total = (0 - Min) + Max;
            double OffsetValue = (0 - Min) + value;
            double Ratio = OffsetValue / Total;
            return this.ClientRectangle.Height - (int)(this.ClientRectangle.Height * Ratio);
        }
    }
}
