using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ShoreSide.Core;

namespace ShoreSide.UI
{
    public partial class PCTWidget : UserControl
    {
        private PCTController controller = null;

        public PCTWidget()
        {
            InitializeComponent();
        }

        protected override void OnResize(EventArgs e)
        {
            try
            {
                outputNeg.Location = new Point(hist.Location.X, outputNeg.Location.Y);
                outputNeg.Size = new Size(hist.Width / 2, outputNeg.Size.Height);

                outputPos.Location = new Point(hist.Location.X + hist.Width / 2, outputPos.Location.Y);
                outputPos.Size = new Size(hist.Width / 2, outputPos.Size.Height);

                outputNeg.Value = 0;
                outputPos.Value = 0;
            }
            catch (Exception)
            { }

            base.OnResize(e);
        }

        public PCTController Controller
        {
            get
            {
                return controller;
            }
            set
            {
                controller = value;
                ReadController();
            }
        }

        public void Cycle()
        {
            if (controller.Output > 0)
            {
                outputNeg.Value = 0;
                outputPos.Value = (int)Math.Min(controller.Output*100, outputPos.Maximum);
            }
            else if (controller.Output < 0)
            {
                outputNeg.Value = (int)Math.Min(Math.Abs(controller.Output)*100, outputNeg.Maximum);
                outputPos.Value = 0;
            }
            else
            {
                outputNeg.Value = 0;
                outputPos.Value = 0;
            }

            labelReference.Text = controller.GetReference().ToString("0.0");
            labelCurrent.Text = controller.Perception().ToString("0.0");
        }

        private void ReadController()
        {
            if (controller == null)
                return;

            labelName.Text = controller.Name;
            numericPGain.Value = (decimal)controller.P_Gain;
            numericDGain.Value = (decimal)controller.D_Gain;
            hist.AddHistogram(new SampleHistogram(controller.Perception), Color.Blue, controller.MinValue, controller.MaxValue);
            hist.AddHistogram(new SampleHistogram(controller.GetReference), Color.Red, controller.MinValue, controller.MaxValue);
            hist.UpdateDelay = TimeSpan.FromMilliseconds(100);
            hist.MoveSpeed = 2;
            numericReference.Value = (decimal)controller.Reference;
            numericReference.Minimum = (decimal)controller.MinValue;
            numericReference.Maximum = (decimal)controller.MaxValue;
            numericReference.Increment = (decimal)((controller.MaxValue - controller.MinValue) / 50);
        }

        private void numericPGain_ValueChanged(object sender, EventArgs e)
        {
            controller.P_Gain = (double)numericPGain.Value;
        }

        private void numericDGain_ValueChanged(object sender, EventArgs e)
        {
            controller.D_Gain = (double)numericDGain.Value;
        }

        private void checkEnabled_CheckedChanged(object sender, EventArgs e)
        {
            controller.Enabled = checkEnabled.Checked;
        }

        private void numericReference_ValueChanged(object sender, EventArgs e)
        {
            controller.Reference = (double)numericReference.Value;
        }
    }
}
